@@ -32,7 +32,7 @@ import pprint
 from keyword import iskeyword
 
 # pycrate_core is used only for basic library-wide functions / variables:
-# log(), python_version, integer_types, str_types
+# log(), integer_types, str_types
 from pycrate_core.utils import *
 from .err               import ASN1Err
 
@@ -42,7 +42,7 @@ from .err               import ASN1Err
 
 def asnlog(msg):
     """
-    customizable logging function for the whole asn1 part
+    customizable logging function for the whole asn1c part
     """
     log(msg)
 
@@ -333,10 +333,16 @@ SYNT_RE_MODULEDEF = re.compile(
     '\s{1,}(DEFINITIONS)\s{1,}')
 SYNT_RE_MODULEREF = re.compile(
     '(?:^|\s{1})(%s){1}\s{0,}(\{[\s\-a-zA-Z0-9\(\)]{1,}\}){0,1}' % _RE_TYPEREF)
+
 SYNT_RE_MODULEFROM = re.compile(
-    '(?:FROM\s{1,})(%s)' \
-    '(?:\s{0,}(\{[\s\-a-zA-Z0-9\(\)]{1,}\})|\s{1,}(%s)(?:\s{1,}%s(?:\s*\{\})?(?:\s{0,},|\s{1,}FROM)|\s{0,}$)){0,1}' \
-    % (_RE_TYPEREF, _RE_IDENT, _RE_WORD))
+    '(?:FROM\s{1,})(%s)\s*' % _RE_TYPEREF)
+SYNT_RE_MODULEFROM_SYM = re.compile(
+    '(%s)(?:\s*\{\s*\}){0,1}(?:\s*,|\s{1,}FROM)' % _RE_WORD)
+SYNT_RE_MODULEFROM_OID = re.compile(
+    '(%s)\s*|(\{[a-zA-Z0-9\(\)\-\s]{4,}\})\s*' % _RE_IDENT)
+SYNT_RE_MODULEFROM_WIT = re.compile(
+    'WITH\s{1,}(SUCCESSORS|DESCENDANTS)\s*')
+
 SYNT_RE_MODULEEXP = re.compile(
     '(?:^|\s{1})EXPORTS((.|\n)*?);')
 SYNT_RE_MODULEIMP = re.compile(
@@ -402,6 +408,8 @@ SYNT_RE_TIMEGENE = re.compile(
 SYNT_RE_CONST_DISPATCH = re.compile(
     '(?:^|\s{1})(INCLUDES)|(SIZE)|(FROM)|(WITH COMPONENTS)|(WITH COMPONENT)|' \
     '(PATTERN)|(SETTINGS)|(CONTAINING)|(ENCODED BY)|(CONSTRAINED BY)')
+SYNT_RE_CONST_EXT = re.compile(
+    ',\s{0,}\.\.\.')
 SYNT_RE_GROUPVERS = re.compile(
     '(?:^|\s{1})[0-9]{1,}\s{0,1}\:')
 
@@ -450,14 +458,14 @@ def scan_for_comments(text=''):
         while text[cur:1+cur] == '-':
             cur += 1
         while True:
-            # move 1 by 1 and find an end-of-comment or end-of-file
+            # move 1 by 1
             if text[cur:1+cur] == '\n' or cur >= len(text):
-                comment = False
+                # end-of-line or end-of-file
                 ret.append((start, cur))
                 cur += 1
                 break
             elif text[cur:2+cur] == '--':
-                comment = False
+                # end-of-comment
                 cur += 2
                 ret.append((start, cur))
                 break
@@ -468,6 +476,38 @@ def scan_for_comments(text=''):
     return ret
 
 
+def scan_for_comments_cstyle(text=''):
+    """
+    returns a list of 2-tuple (start offset, end offset) for each ASN.1 comment
+    in C-style found in text
+    """
+    ret = []
+    cur = 0
+    next = text.find('/*')
+    while next >= 0:
+        cur += next
+        # start of comment
+        start = cur
+        # move cursor forward to reach the end of comment
+        cur += 2
+        while True:
+            # move 1 by 1 and find an end-of-comment or end-of-file
+            if cur >= len(text):
+                # end-of-file
+                ret.append((start, cur))
+                break
+            elif text[cur:2+cur] == '*/':
+                # end-of-comment
+                cur += 2
+                ret.append((start, cur))
+                break
+            else:
+                cur += 1
+        # find the next comment
+        next = text[cur:].find('/*')
+    return ret
+
+
 def clean_text(text=''):
     """
     processes text to: 
@@ -475,6 +515,9 @@ def clean_text(text=''):
         replace tab with space
         remove duplicated spaces
     """
+    # WARNING: this routine for text cleanup, as it is applied early in the text
+    # processing, may mess up ASN.1 string values
+    #
     # remove comments
     comments = scan_for_comments(text)
     if comments:
@@ -487,6 +530,16 @@ def clean_text(text=''):
         defins.append( text[start:len(text)] )
         text = ''.join(defins)
     #
+    # remove C-style comments
+    comments = scan_for_comments_cstyle(text)
+    if comments:
+        start, defins = 0, []
+        for (so, eo) in comments:
+            defins.append( text[start:so] )
+            start = eo
+        defins.append( text[start:len(text)] )
+        text = ''.join(defins)
+    #
     # replace tab with space
     text = text.replace('\t', ' ')
     # remove duplicated CR
@@ -804,16 +857,36 @@ def extract_set(text=''):
               .format(valset)))
 
 
-#------------------------------------------------------------------------------#
-# following definitions are (yet) unused
-#------------------------------------------------------------------------------#
+def extract_from_import(text=''):
+    """
+    extracts the module name, reference and / or OID set after a FROM import
+    statement, test `text` argument must start with the FROM keyword
+    
+    returns a 2-tuple with
+        integer: length of the text containing the whole FROM statement
+        dict: with "name", "oid", "oidref" and "with" keys
+    """
+    m = SYNT_RE_MODULEFROM.match(text)
+    assert(m)
+    cur = m.end()
+    ret = {'name': m.group(1), 'oid': None, 'oidref': None, 'with': None}
+    # check if we stop or continue with an OID value or OID reference
+    if SYNT_RE_MODULEFROM_SYM.match(text[cur:]) or not text[cur:]:
+        return cur, ret
+    m = SYNT_RE_MODULEFROM_OID.match(text[cur:])
+    assert(m)
+    cur += m.end()
+    assert(None in m.groups())
+    if m.group(1):
+        ret['oidref'] = m.group(1)
+    else:
+        ret['oid'] = m.group(2)
+    # check if there is a final WITH stmt
+    m = SYNT_RE_MODULEFROM_WIT.match(text[cur:])
+    if m:
+        ret['with'] = m.group(1)
+        cur += m.end()
+    # final control
+    assert(SYNT_RE_MODULEFROM_SYM.match(text[cur:]) or not text[cur:])
+    return cur, ret
 
-#------------------------------------------------------------------------------#
-# class syntax processing routines
-#------------------------------------------------------------------------------#
-
-def class_syntax_gidbl(gidbl, gidcur):
-    for gid in gidbl:
-        if gid == gidcur[:len(gid)]:
-            return True
-    return False
