@@ -1009,7 +1009,7 @@ class ASN1Obj(Element):
     def get_val(self):
         return self._val
     
-    def get_val_paths(self, curpath=[], paths=[]):
+    def get_val_paths(self, curpath=None, paths=None):
         """
         returns the list of paths of each individual basic value set into self
         
@@ -1022,6 +1022,10 @@ class ASN1Obj(Element):
         if self._val is None:
             return []
         #
+        if curpath is None:
+            curpath = []
+        if paths is None:
+            paths = []
         if self.TYPE in (TYPE_CHOICE, TYPE_ANY, TYPE_OPEN) or \
         self.TYPE in (TYPE_BIT_STR, TYPE_OCT_STR) and \
         isinstance(self._val, tuple) and \
@@ -1078,7 +1082,95 @@ class ASN1Obj(Element):
             paths.append( (curpath[:], self._val) )
         #
         return paths[:]
-    
+
+    def get_val_jer_paths(self, curpath=None, jercurpath=None, paths=None):
+        """
+        returns both the list of internal paths and JER paths of each individual 
+        basic value set into self
+
+        Args:
+            None
+
+        Returns:
+            list of 3-tuple: (path_to_basic_value, jer_path_to_basic_value, basic_value)
+        """
+        if self._val is None:
+            return []
+        #
+        if curpath is None:
+            curpath = []
+        if jercurpath is None:
+            jercurpath = []
+        if paths is None:
+            paths = []
+        if self.TYPE in (TYPE_CHOICE, TYPE_ANY, TYPE_OPEN) or \
+        self.TYPE in (TYPE_BIT_STR, TYPE_OCT_STR) and \
+        isinstance(self._val, tuple) and \
+        isinstance(self._val[0], str_types):
+            # value is (component_name, component_value)
+            curpath.append( self._val[0] )
+            # open types are skipped in JER paths
+            if self.TYPE not in (TYPE_OPEN):
+                jercurpath.append( self._val[0] )
+            if self._val[0][:5] in ('_ext_', '_unk_'):
+                # take care of unknown / extended objects
+                paths.append( (curpath[:], jercurpath[:], self._val[1]) )
+            else:
+                Comp = self.get_at( [self._val[0]] )
+                _comp_val = Comp._val
+                Comp._val = self._val[1]
+                paths = Comp.get_val_jer_paths(curpath[:], jercurpath[:], paths[:])
+                Comp._val = _comp_val
+            del curpath[-1]
+            if self.TYPE not in (TYPE_OPEN):
+                del jercurpath[-1]
+        #
+        elif self.TYPE in (TYPE_SEQ, TYPE_SET, TYPE_EXT, TYPE_EMB_PDV, TYPE_CHAR_STR):
+            # value is dict {component_name: component_value)
+            val_ids = list(self._val.keys())
+            for comp_name in self._cont:
+                if comp_name in val_ids[:]:
+                    curpath.append( comp_name )
+                    jercurpath.append( comp_name )
+                    comp_val = self._val[comp_name]
+                    Comp = self._cont[comp_name]
+                    _comp_val = Comp._val
+                    Comp._val = comp_val
+                    paths = Comp.get_val_jer_paths(curpath[:], jercurpath[:], paths[:])
+                    Comp._val = _comp_val
+                    del curpath[-1]
+                    del jercurpath[-1]
+                    val_ids.remove(comp_name)
+            if val_ids:
+                # take care of remaining unknown / extension objects
+                val_ids.sort()
+                for comp_name in val_ids:
+                    curpath.append( comp_name )
+                    jercurpath.append( comp_name )
+                    assert( comp_name[:5] in ('_ext_', '_unk_') )
+                    paths.append( (curpath[:], jercurpath[:], self._val[comp_name][1]) )
+                    del curpath[-1]
+                    del jercurpath[-1]
+        #
+        elif self.TYPE in (TYPE_SEQ_OF, TYPE_SET_OF):
+            # value is a list of component_value
+            Comp = self._cont
+            _comp_val = Comp._val
+            for i, val in enumerate(self._val):
+                Comp._val = val
+                curpath.append( i )
+                jercurpath.append( i )
+                paths = Comp.get_val_jer_paths(curpath[:], jercurpath[:], paths[:])
+                del curpath[-1]
+                del jercurpath[-1]
+            Comp._val = _comp_val
+        #
+        elif self.TYPE in TYPES_BASIC:
+            # basic value reached
+            paths.append( (curpath[:], jercurpath[:], self._val) )
+        #
+        return paths[:]
+
     def get_val_at(self, path):
         """
         returns the value into self at the given relative path
@@ -1446,27 +1538,48 @@ class ASN1Obj(Element):
     # conversion between internal value and ASN.1 BER encoding
     ###
     
-    def __to_ber_codec_set(self):
-        # 0) enables BER length encoding options to be set by field
-        if hasattr(self, '_BER_ENC_LLONG'):
-            ber_enc_llong = ASN1CodecBER.ENC_LLONG
-            ASN1CodecBER.ENC_LLONG = self._BER_ENC_LLONG
-        else:
-            ber_enc_llong = None
-        if hasattr(self, '_BER_ENC_LUNDEF'):
-            # this is only for constructed types,
-            # but it is easier to handle it here globally
-            ber_enc_lundef = ASN1CodecBER.ENC_LUNDEF
-            ASN1CodecBER.ENC_LUNDEF = self._BER_ENC_LUNDEF
-        else:
-            ber_enc_lundef = None
-        return (ber_enc_llong, ber_enc_lundef)
+    # for BER encoding, we can enable per-object encoding options
+    # set this to False to disable per-object encoding options
+    _BER_ENC_OPT  = True
     
-    def __to_ber_codec_unset(self, *ber_enc_args):
-        if ber_enc_args[0] is not None:
-            ASN1CodecBER.ENC_LLONG = ber_enc_args[0]
-        if ber_enc_args[1] is not None:
-            ASN1CodecBER.ENC_LUNDEF = ber_enc_args[1]
+    # following are the different BER encoding options supported
+    # see codecs.ASN1CodecBER class for further description
+    _BER_ENC_ARGS = (
+        # for tag prefix
+        'TAG_LEXT',
+        # for length prefix
+        'LLONG',
+        'LUNDEF',
+        # for DEFAULT values
+        'DEF_CANON',
+        # for BOOLEAN object
+        'BOOLTRUE',
+        # for BIT / OCTET STRING object
+        'BSTR_FRAG',
+        'OSTR_FRAG',
+        # for *OID* object
+        'OID_LEXT',
+        # for *Time* object
+        'TIME_CANON',
+        )
+    
+    def __to_ber_codec_set(self):
+        ber_enc_args = {}
+        for arg in self._BER_ENC_ARGS:
+            attr_g, attr_l = 'ENC_%s' % arg, '_BER_ENC_%s' % arg
+            if hasattr(self, attr_l):
+                # save the global attribute for restoration
+                # after object has been encoded
+                ber_enc_args[attr_g] = getattr(ASN1CodecBER, attr_g)
+                # set the global attribute to the object's local value
+                setattr(ASN1CodecBER, attr_g, getattr(self, attr_l))
+        return ber_enc_args
+    
+    def __to_ber_codec_unset(self, ber_enc_args):
+        for k, v in ber_enc_args.items():
+            setattr(ASN1CodecBER, k, v)
+    
+    # std BER decoding / encoding routines
     
     def _from_ber(self, char, TLV):
         # 1) decode the tag chain
@@ -1509,7 +1622,8 @@ class ASN1Obj(Element):
     
     def _to_ber(self):
         # 0) set potential BER codec locals
-        _ber_enc_args = self.__to_ber_codec_set()
+        if self._BER_ENC_OPT:
+            _ber_enc_args = self.__to_ber_codec_set()
         #
         # 1) encode the most inner TLV part
         pc, lval, V = self._encode_ber_cont()
@@ -1518,11 +1632,20 @@ class ASN1Obj(Element):
             # we only have the inner encoding
             ret = V
         else:
-            TLV = ASN1CodecBER.encode_tag(self._tagc[-1][0], pc, self._tagc[-1][1])
-            TLV.extend( ASN1CodecBER.encode_len(lval) )
-            TLV.extend( V )
-            if lval == -1:
+            if pc == 1 and ASN1CodecBER.ENC_LUNDEF:
+                # fragmented String
+                TLV = ASN1CodecBER.encode_tag(self._tagc[-1][0], pc, self._tagc[-1][1])
+                TLV.extend( ASN1CodecBER.encode_len(-1) )
+                TLV.extend( V )
                 TLV.append( (T_BYTES, b'\0\0', 16) )
+            else:
+                TLV = ASN1CodecBER.encode_tag(self._tagc[-1][0], pc, self._tagc[-1][1])
+                TLV.extend( ASN1CodecBER.encode_len(lval) )
+                TLV.extend( V )
+                if lval == -1:
+                    # append an EOC marker after the value
+                    TLV.append( (T_BYTES, b'\0\0', 16) )
+            #
             # 2) encode the outer part of the object, i.e. the rest of the tag chain
             if len(self._tagc) > 1:
                 GEN = [TLV]
@@ -1546,7 +1669,8 @@ class ASN1Obj(Element):
                 ret = TLV
         #
         # 2) restore potential BER encoder globals
-        self.__to_ber_codec_unset(*_ber_enc_args)
+        if self._BER_ENC_OPT:
+            self.__to_ber_codec_unset(_ber_enc_args)
         return ret
     
     def to_ber(self, val=None):
@@ -1626,7 +1750,8 @@ class ASN1Obj(Element):
     
     def _to_ber_ws(self):
         # 0) set potential BER codec locals
-        _ber_enc_args = self.__to_ber_codec_set()
+        if self._BER_ENC_OPT:
+            _ber_enc_args = self.__to_ber_codec_set()
         #
         # 1) encode the most inner TLV part
         pc, lval, V = self._encode_ber_cont_ws()
@@ -1647,6 +1772,12 @@ class ASN1Obj(Element):
                         ASN1CodecBER.encode_tag_ws(self._tagc[-1][0], pc, self._tagc[-1][1]),
                         ASN1CodecBER.encode_len_ws(lval),
                         V))
+                if lval == -1:
+                    # append an EOC marker after the value
+                    TLV.extend((
+                        ASN1CodecBER.encode_tag_ws(0, 0, 0),
+                        ASN1CodecBER.encode_len_ws(0)
+                        ))
             # 2) encode the outer part of the object, i.e. the rest of the tag chain
             if len(self._tagc) > 1:
                 if ASN1CodecBER.ENC_LUNDEF:
@@ -1670,7 +1801,8 @@ class ASN1Obj(Element):
         self._struct = TLV
         #
         # 2) restore potential BER encoder globals
-        self.__to_ber_codec_unset(*_ber_enc_args)
+        if self._BER_ENC_OPT:
+            self.__to_ber_codec_unset(_ber_enc_args)
         return TLV
     
     def to_ber_ws(self, val=None):
@@ -1875,9 +2007,9 @@ class ASN1Obj(Element):
             char = Charpy(buf)
         else:
             char = buf
-
+        #
         self._from_oer(char)
-
+        #
         if self._SAFE_BND:
             self._safechk_bnd(self._val)
 
@@ -1887,9 +2019,9 @@ class ASN1Obj(Element):
             char = Charpy(buf)
         else:
             char = buf
-
+        #
         self._from_oer_ws(char)
-
+        #
         if self._SAFE_BND:
             self._safechk_bnd(self._val)
 
