@@ -100,9 +100,9 @@ Single value: int 0
         if not isinstance(vbnd, tuple):
             raise(ASN1BERDecodeErr('{0}: invalid NULL constructed structure'\
                   .format(self.fullname())))
-        if vbnd[1] != vbnd[0]:
+        elif vbnd[1] != vbnd[0]:
             raise(ASN1BERDecodeErr('{0}: invalid NULL length, {1!r}'\
-                  .format(self.fullname(), lval)))
+                  .format(self.fullname(), vbnd)))
         self._val = 0
         return Buf('V', val=b'', bl=0)
     
@@ -110,9 +110,9 @@ Single value: int 0
         if not isinstance(vbnd, tuple):
             raise(ASN1BERDecodeErr('{0}: invalid NULL constructed structure'\
                   .format(self.fullname())))
-        if vbnd[1] != vbnd[0]:
+        elif vbnd[1] != vbnd[0]:
             raise(ASN1BERDecodeErr('{0}: invalid NULL length, {1!r}'\
-                  .format(self.fullname(), lval)))
+                  .format(self.fullname(), vbnd)))
         self._val = 0
     
     def _encode_ber_cont_ws(self):
@@ -331,7 +331,7 @@ Specific attribute:
     TYPE  = TYPE_INT
     TAG   = 2
     
-    _ASN_RE = re.compile('\-{0,1}[0-9]{1,}')
+    _ASN_RE = re.compile(r'\-{0,1}[0-9]{1,}')
     
     def _safechk_val(self, val):
         if isinstance(val, str_types):
@@ -689,12 +689,12 @@ Specific attribute:
     TAG   = 9
     
     _ASN_RE = re.compile(
-    '((\-{0,1}[0-9]{1,}){1}(?:\.([0-9]{0,})){0,1}(?:[eE](\-{0,1}[0-9]{1,})){0,1})|'\
-    '(\{\s{0,}mantissa\s{1,}(\-{0,1}[0-9]{1,})\s{0,},\s{0,}base\s{1,}(2|10)\s{0,},\s{0,}exponent\s{1,}(\-{0,1}[0-9]{1,})\s{0,}\})|'\
-    '((?:PLUS\-INFINITY)|(?:MINUS\-INFINITY)|(?:NOT-A-NUMBER))')
-    _NR1_RE = re.compile('[ 0]{0,}[-+]{0,1}[0-9]{1,}')
-    _NR2_RE = re.compile('[ 0]{0,}([-+]{0,1}[0-9]{0,})[\.,]{1}([0-9]{0,})')
-    _NR3_RE = re.compile('[ 0]{0,}([-+]{0,1}[0-9]{0,})[\.,]{1}([0-9]{0,})[eE]([-+]{0,1}[0-9]{1,})')
+    r'((\-{0,1}[0-9]{1,}){1}(?:\.([0-9]{0,})){0,1}(?:[eE](\-{0,1}[0-9]{1,})){0,1})|'\
+    r'(\{\s{0,}mantissa\s{1,}(\-{0,1}[0-9]{1,})\s{0,},\s{0,}base\s{1,}(2|10)\s{0,},\s{0,}exponent\s{1,}(\-{0,1}[0-9]{1,})\s{0,}\})|'\
+    r'((?:PLUS\-INFINITY)|(?:MINUS\-INFINITY)|(?:NOT-A-NUMBER))')
+    _NR1_RE = re.compile(r'[ 0]{0,}[-+]{0,1}[0-9]{1,}')
+    _NR2_RE = re.compile(r'[ 0]{0,}([-+]{0,1}[0-9]{0,})[\.,]{1}([0-9]{0,})')
+    _NR3_RE = re.compile(r'[ 0]{0,}([-+]{0,1}[0-9]{0,})[\.,]{1}([0-9]{0,})[eE]([-+]{0,1}[0-9]{1,})')
     
     _ASN_LUT = {'MINUS-INFINITY': (-1, None, None),
                 'PLUS-INFINITY' : ( 1, None, None),
@@ -703,7 +703,7 @@ Specific attribute:
                 ( 1, None, None): 'PLUS-INFINITY',
                 ( 0, None, None): 'NOT-A-NUMBER'}
     
-    _JER_RE = re.compile('[ 0]{0,}([-+]{0,1}[0-9]{0,})(?:[\.,]{1}([0-9]{0,})){0,1}[eE]([-+]{0,1}[0-9]{1,})')
+    _JER_RE = re.compile(r'[ 0]{0,}([-+]{0,1}[0-9]{0,})(?:[\.,]{1}([0-9]{0,})){0,1}[eE]([-+]{0,1}[0-9]{1,})')
     
     def _safechk_val(self, val):
         self._safechk_val_real(val)
@@ -786,10 +786,7 @@ Specific attribute:
                 # base 10: character string encoding
                 if B0 == 1:
                     # NR1 encoding, simple whole numbers
-                    if python_version < 3:
-                        m = self._NR1_RE.match( bytes[1:] )
-                    else:
-                        m = self._NR1_RE.match( str(bytes[1:], 'ascii') )
+                    m = self._NR1_RE.match( str(bytes[1:], 'ascii') )
                     if not m:
                         raise(ASN1BERDecodeErr('{0}: invalid REAL base 10 NR1 encoding, {1!r}'\
                               .format(self.fullname(), bytes[1:])))
@@ -797,10 +794,7 @@ Specific attribute:
                     self._val = (mant, 10, 0)
                 elif B0 == 2:
                     # NR2 encoding, requires a decimal mark
-                    if python_version < 3:
-                        m = self._NR2_RE.match( bytes[1:] )
-                    else:
-                        m = self._NR2_RE.match( str(bytes[1:], 'ascii') )
+                    m = self._NR2_RE.match( str(bytes[1:], 'ascii') )
                     if not m:
                         raise(ASN1BERDecodeErr('{0}: invalid REAL base 10 NR2 encoding, {1!r}'\
                               .format(self.fullname(), bytes[1:])))
@@ -812,10 +806,7 @@ Specific attribute:
                     self._val = (int(i+d), 10, e)
                 elif B0 == 3:
                     # NR3 encoding, requires the exponent notation
-                    if python_version < 3:
-                        m = self._NR3_RE.match( bytes[1:] )
-                    else:
-                        m = self._NR3_RE.match( str(bytes[1:], 'ascii') )
+                    m = self._NR3_RE.match( str(bytes[1:], 'ascii') )
                     if not m:
                         raise(ASN1BERDecodeErr('{0}: invalid REAL base 10 NR3 encoding, {1!r}'\
                               .format(self.fullname(), bytes[1:])))
@@ -890,21 +881,12 @@ Specific attribute:
                     raise(ASN1BEREncodeErr('{0}: invalid REAL base 10 encoding NR1 for decimal value'\
                           .format(self.fullname())))
                 i = self._val[0] * (10**self._val[2])
-                if python_version < 3:
-                    return '\x01' + \
-                           ASN1CodecBER.ENC_REALNR1_SPA * ' ' + \
-                           ASN1CodecBER.ENC_REALNR1_ZER * '0' + \
-                           str(i)
-                else:
-                    return b'\x01' + \
-                           ASN1CodecBER.ENC_REALNR1_SPA * b' ' + \
-                           ASN1CodecBER.ENC_REALNR1_ZER * b'0' + \
-                           bytes(str(i), 'ascii')
+                return b'\x01' + \
+                       ASN1CodecBER.ENC_REALNR1_SPA * b' ' + \
+                       ASN1CodecBER.ENC_REALNR1_ZER * b'0' + \
+                       bytes(str(i), 'ascii')
             elif ASN1CodecBER.ENC_REALNR == 2:
-                if python_version < 3:
-                    i = bytes(self._val[0])
-                else:
-                    i = bytes(str(self._val[0]), 'ascii')
+                i = bytes(str(self._val[0]), 'ascii')
                 if self._val[2] >= 0:
                     # we need to add trailing zero
                     i += self._val[2] * b'0'
@@ -926,20 +908,14 @@ Specific attribute:
                        ASN1CodecBER.ENC_REALNR2_ZERTRAIL * b'0'
             else:
                 #ASN1CodecBER.ENC_REALNR == 3
-                if python_version < 3:
-                    i, e = bytes(self._val[0]), self._val[2]
-                else:
-                    i, e = bytes(str(self._val[0]), 'ascii'), self._val[2]
+                i, e = bytes(str(self._val[0]), 'ascii'), self._val[2]
                 # remove trailing zero
                 while i[-1:] == b'0':
                     i = i[:-1]
                     e += 1
                 if e == 0:
                     e = b'+0'
-                elif python_version < 3:
-                    e = bytes(e)
-                else:
-                    e = bytes(str(e), 'ascii')
+                e = bytes(str(e), 'ascii')
                 return b'\x03' + i + b'.E' + e
         else:
             # base 2, encoding the CER / DER way:
@@ -957,16 +933,10 @@ Specific attribute:
             LE = int_bytelen(E)
             E = int_to_bytes(E, 8*LE)
             N = uint_to_bytes(m, 8*uint_bytelen(m))
-            if python_version < 3:
-                if LE > 3:
-                    return chr(0x80 + (S<<6) + 3) + uint_to_bytes(LE) + E + N
-                else:
-                    return chr(0x80 + (S<<6) + LE-1) + E + N
+            if LE > 3:
+                return bytes((0x80 + (S<<6) + 3, )) + uint_to_bytes(LE) + E + N
             else:
-                if LE > 3:
-                    return bytes((0x80 + (S<<6) + 3, )) + uint_to_bytes(LE) + E + N
-                else:
-                    return bytes((0x80 + (S<<6) + LE-1, )) + E + N
+                return bytes((0x80 + (S<<6) + LE-1, )) + E + N
     
     def _decode_ber_cont_ws(self, char, vbnd):
         char._cur, char._len_bit = vbnd[0], vbnd[1]
@@ -1591,9 +1561,9 @@ class _OID(ASN1Obj):
     
     # this class implements the methods that are common to OID and REL_OID    
     
-    _ASN_RE = re.compile('\{([a-zA-Z0-9\-\(\)\s]{1,})\}')
+    _ASN_RE = re.compile(r'\{([a-zA-Z0-9\-\(\)\s]{1,})\}')
     _ASN_RE_COMP = re.compile(
-    '([0-9]{1,})|(?:([a-zA-Z]{1}[a-zA-Z0-9\-]{0,})\s{0,}(?:\(([0-9]{1,})\)){0,1})')
+    r'([0-9]{1,})|(?:([a-zA-Z]{1}[a-zA-Z0-9\-]{0,})\s{0,}(?:\(([0-9]{1,})\)){0,1})')
     
     # _ASN_WASC potentially add the name of the OID in ascii in comment
     # when returned by _to_asn1() 
@@ -1611,7 +1581,7 @@ class _OID(ASN1Obj):
     def _from_asn1(self, txt):
         m = self._ASN_RE.match(txt)
         if m:
-            txtval = txt[m.start()+1:m.end()-1]
+            txtval = txt[m.start()+1:m.end()-1].strip()
             _val = []
             txt = txt[m.end():].strip()
             m = self._ASN_RE_COMP.match(txtval)
@@ -1805,10 +1775,7 @@ Single value: Python tuple of int
             for f in fact[:-1]:
                 by.append( 0x80 + f )
             by.append( fact[-1] )
-        if python_version > 2:
-            return bytes(by)
-        else:
-            return ''.join(map(chr, by))    
+        return bytes(by)   
 
 
 class REL_OID(_OID):
@@ -1856,8 +1823,5 @@ Single value: Python tuple of int
             for f in fact[:-1]:
                 by.append( 0x80 + f )
             by.append( fact[-1] )
-        if python_version > 2:
-            return bytes(by)
-        else:
-            return ''.join(map(chr, by))    
+        return bytes(by)
 
