@@ -101,7 +101,7 @@ Single value: Python 2-tuple
     # this enables object's table constraint lookup for OPEN types when decoding it
     _TAB_LUT = True
     
-    _ASN_RE = re.compile('(?:\'([\s01]{0,})\'B)|(?:\'([\s0-9A-F]{0,})\'H)')
+    _ASN_RE = re.compile(r'(?:\'([\s01]{0,})\'B)|(?:\'([\s0-9A-F]{0,})\'H)')
     
     def _get_val_obj(self, ref):
         const_tr = self._get_const_tr()
@@ -109,7 +109,9 @@ Single value: Python 2-tuple
             if ref in const_tr:
                 return const_tr[ref]
             elif ref in _ASN1ObjBasicLUT:
-                return _ASN1ObjBasicLUT[ref]()
+                obj = _ASN1ObjBasicLUT[ref]()
+                obj._tagc = [(0, obj.TAG)]
+                return obj
             else:
                 raise(ASN1ObjErr('{0}: invalid object reference, {1!r}'\
                       .format(self.fullname(), ref)))
@@ -197,11 +199,11 @@ Single value: Python 2-tuple
                 ident = '_unk_004'
             if grp[0] is not None:
                 # BSTRING
-                bs = re.subn('\s{1,}', '', grp[0])[0]
+                bs = re.subn(r'\s{1,}', '', grp[0])[0]
                 self._val = (ident, uint_to_bytes(int(bs, 2), len(bs)))
             else:
                 # HSTRING
-                hs = re.subn('\s{1,}', '', grp[1])[0]
+                hs = re.subn(r'\s{1,}', '', grp[1])[0]
                 if len(hs)%2:
                     self._val = (ident, unhexlify(hs + '0'))
                 else:
@@ -212,7 +214,7 @@ Single value: Python 2-tuple
             # TODO: must implement the Module.value notation to compare to tuples
             # into const_tr, in addition to simple object name
             const_tr_keys = [name for name in self._get_const_tr() if isinstance(name, str_types)]
-            m = re.match('\s{0,}:|'.join(const_tr_keys) + '\s{0,}:', txt)
+            m = re.match(r'\s{0,}:|'.join(const_tr_keys) + r'\s{0,}:', txt)
             if m is not None:
                 ident = m.group().split(':')[0].strip()
                 txt = txt[m.end():].strip()
@@ -230,10 +232,7 @@ Single value: Python 2-tuple
         if isinstance(self._val[0], str_types):
             if self._val[0][:5] == '_unk_':
                 # HSTRING
-                if python_version >= 3:
-                    return '\'%s\'H' % hexlify(self._val[1]).decode('ascii').upper()
-                else:
-                    return '\'%s\'H' % hexlify(self._val[1]).upper()
+                return '\'%s\'H' % hexlify(self._val[1]).decode('ascii').upper()
             else:
                 ident = self._val[0]
                 Obj   = self._get_val_obj(self._val[0])
@@ -642,11 +641,11 @@ Single value: Python 2-tuple
                 Obj = self._get_val_obj(self._val[0])
             Obj._val = self._val[1]
             return Obj._to_jval()
-
+    
     ###
     # conversion between internal value and ASN.1 OER/COER encoding
     ###
-
+    
     def _from_oer(self, char):
         # try to get a defined object from a table constraint
         if self._TAB_LUT and self._const_tab and self._const_tab_at:
@@ -671,7 +670,7 @@ Single value: Python 2-tuple
         #
         val_bytes = ASN1CodecOER.decode_open_type(char)
         val = Obj.from_oer(val_bytes) if (Obj is not None) else val_bytes
-
+        
         if Obj is None:
             if self._const_val:
                 asnlog('OPEN._from_per: %s, potential type constraint(s) available but unused' \
@@ -684,7 +683,7 @@ Single value: Python 2-tuple
             else:
                 self._val = (Obj.TYPE, val)
         return
-
+    
     def _from_oer_ws(self, char):
         # try to get a defined object from a table constraint
         if self._TAB_LUT and self._const_tab and self._const_tab_at:
@@ -720,7 +719,7 @@ Single value: Python 2-tuple
             else:
                 self._val = (Obj.TYPE, val)
         self._struct = Envelope(self._name, GEN=tuple(GEN))
-
+    
     def _to_oer(self):
         if isinstance(self._val[0], ASN1Obj):
             Obj = self._val[0]
@@ -730,7 +729,7 @@ Single value: Python 2-tuple
             Obj = self._get_val_obj(self._val[0])
         Obj._val = self._val[1]
         return ASN1CodecOER.encode_open_type(Obj.to_oer())
-
+    
     def _to_oer_ws(self):
         if isinstance(self._val[0], ASN1Obj):
             Obj = self._val[0]
