@@ -39,7 +39,7 @@ from .err                import *
 from pycrate_asn1c.utils import clean_text
 
 # pycrate_core is used for basic library-wide functions / variables
-# (i.e. log(), python_version, integer_types, bytes_types, str_types)
+# (i.e. log(), integer_types, bytes_types, str_types)
 # and for encoding / decoding routines
 from pycrate_core.utils  import *
 from pycrate_core.utils  import TYPE_BYTES   as T_BYTES
@@ -332,7 +332,7 @@ def extract_charstr(text=''):
                 # no end-of-charstr found
                 return text, None
             else:
-                return '', re.subn('\s{0,}\n\s{0,}', '', text[1:-1])[0]
+                return '', re.subn(r'\s{0,}\n\s{0,}', '', text[1:-1])[0]
         
         # 2) finding a double-quote
         if text[cur:1+cur] == '"':
@@ -349,7 +349,7 @@ def extract_charstr(text=''):
                     else:
                         # end of charstr
                         return text[1+cur:].strip(), \
-                               re.subn('\s{0,}\n\s{0,}', '', text[1:cur])[0]
+                               re.subn(r'\s{0,}\n\s{0,}', '', text[1:cur])[0]
             else:
                 # 2.2) escape cursor not set
                 if text[1+cur:2+cur] == '"':
@@ -358,14 +358,14 @@ def extract_charstr(text=''):
                 else:
                     # end of charstr
                     return text[1+cur:].strip(), \
-                           re.subn('\s{0,}\n\s{0,}', '', text[1:cur])[0]
+                           re.subn(r'\s{0,}\n\s{0,}', '', text[1:cur])[0]
 
 
-printable_str = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'\
-                '!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~ \t\n\r\x0b\x0c'
+_printable_str = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'\
+                 '!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~ \t\n\r\x0b\x0c'
 
 def is_printable(w):
-    return all(c in printable_str for c in w)
+    return all(c in _printable_str for c in w)
 
 #------------------------------------------------------------------------------#
 # integer factorization and rounding routine
